#!/usr/bin/perl

################################################################
# A perl script to download ecmwf era-15                       #
# data from the badc and optionally run it through convsh      #
# to convert the resultant files to netcdf format	       #
# By: Dave Sproson	                                       #
# Last modified: 19/07/2010				       #
################################################################

use strict;
use warnings;
use Switch;
use Net::FTP;
use Cwd;

#== Set period over which to download data ==#
my $day_start = 1;
my $day_end = 5;
my $month_start = 1;
my $month_end = 1;
my $year_start = 1980;
my $year_end = 1980;
my @hours = ("00","06","12","18");
#============================================#

#================ Local info ================#
our $output_dir = "~";	
our $convert = 1;
our $nc_filename = "ERA.nc";
our $delete_grib = 1;
#============================================#

#=============== Dataset info ===============#
my $stream = "gisf";
# Character 1: g = Gaussian N80; s = Spectral T106; l = gridded (2.5)
# Character 2: i = initialised; p = uninitialised
# Characters 3-4: sf = surface; pr = pressure levs; ua = upper air
# Valid streams: lisf,lipr,gisf,giua,gpsf,gpua,sipr,siua,sppr,spua
#============================================#

#====== BADC server/login credentials =======#
our $username = "";
our $password = "";
#============================================#


###################################################
# DO NOT EDIT BELOW HERE UNLESS YOU KNOW WHAT YOU #
# ARE DOING!!                                     #
###################################################

#== Perform some conditional checks to ensure time ranges are valid ==#
die "Error: incompatible convert and delete_grib options!" if (!$convert && $delete_grib);
mkdir $output_dir, 0777 unless -d $output_dir or die $!;
#=====================================================================#

#push(@fields,"lsm");

my @years = ($year_start .. $year_end);
my @months;
my @days;

my $month_from;
my $month_to;
my $day_from;
my $day_to;

our $ftp;
makeFTPConnection();

foreach my $year (@years) {
  if($year==$year_start) {
    $month_from = $month_start;
  } else {
    $month_from = 1;
  }
  if($year==$year_end) {
    $month_to = $month_end;
  } else {
    $month_to = 12;
  }
  @months = ($month_from .. $month_to);
  
  
  foreach my $month (@months) {
    if($year==$year_start && $month==$month_start) {
      $day_from = $day_start;
    } else {
      $day_from = 1;
    }
    if($year==$year_end && $month==$month_end) {
      $day_to = $day_end;
    } else {
      switch($month) {
	case [2] {
	  if($year%4==0) {
	    $day_to = 29;
	  } else {
	    $day_to = 28;
	  }
	}
	case [4,6,9,11] {$day_to = 30;}
	else {$day_to = 31;}
	}
    }
    @days = ($day_from .. $day_to);
    
    $month = "0$month" if(length($month)<2);
    
    foreach my $day (@days) {
      
      $day = "0$day" if(length($day)<2);
      
      foreach my $hour (@hours) {
	  getGribFile($year,$month,$day,$hour,$stream);
      }
      
    }
    
  }
    
}

$ftp->quit;

my $working_dir = getcwd;
my @convert_to_netCDF = ("conv2nc.tcl -i $output_dir*.grb -o $output_dir$nc_filename");

if($convert) {
  if(system(@convert_to_netCDF)==0) {
    my @lof = ($output_dir."*.grb");
    if($delete_grib) {
      unlink(<@lof>) or warn "$!";
    }
  } else {
    print "NetCDF conversion may have failed, not removing grib files.";
  }
}



#############################################################################################




sub getGribFile {
  my ($year,$month,$day,$hour,$stream) = @_;  
    

  my $mode;
  switch(substr($stream,1,1)) {
    case["i"] {$mode = "init";}
    case["p"] {$mode = "uninit";}
    else { die "$stream is a invalid stream specification (".substr($stream,1,1).")"}
  }
  my $grid;
  
  
  switch(substr($stream,0,1)) {
    case["l"] { $grid = "gridded_2.5"; $mode="";}
    case["g"] { $grid = "N80";}
    case["s"] { $grid = "T106";}
    else { die "$stream is a invalid stream specification (".substr($stream,0,0).")"}
  }

  
  my $fileToGet = "/badc/ecmwf-era/data/$mode/$grid/$year/$stream".substr($year,2,3)."$month/$stream".substr($year,2,3)."$month$day$hour";
  print "Getting $fileToGet\n";
  $ftp->get($fileToGet,"$output_dir$stream$year$month$day$hour".".grb")
    or do {print " --> Warning: ", $ftp->message."\n";}
    
}

sub makeFTPConnection {
  my $server = "ftp.badc.rl.ac.uk";
  
  $ftp = Net::FTP->new($server, Debug => 0)
    or die "Error: could not make ftp connection to $server.\nMessage: $@";
    
  $ftp->login($username,$password)
    or die "Error: could not login to $server with $username/$password: ", $ftp->message;
    
  $ftp->binary;
}

	

