#!/usr/bin/perl

################################################################
# A perl script to download ukmo-nae                           #
# data from the badc and optionally run it through convsh      #
# to convert the resultant files to netcdf format	       #
# By: Dave Sproson	                                       #
# Last modified: 19/07/2010				       #
################################################################


use strict;
use warnings;
use Switch;
use Net::FTP;
use Cwd;
use Time::HiRes;

#== Set period over which to download data ==#
my $day_start = 1;
my $day_end = 10;
my $month_start = 1;
my $month_end = 1;
my $year_start = 2007;
my $year_end = 2007;
my @hours = ("00");
#============================================#

#================ Local info ================#
our $output_dir = "~";
our $convert = 1;
our $nc_filename = "UKMO_NAE.nc";
our $delete_pp = 1;
#============================================#

#=============== Dataset info ===============#
my $stream = "fp";
my @steps = ("01");
my @fields = ("16202");
# Streams: 	f = forecast, a = analysis
#		m = model levs, p = pressure levs
# Steps:	Forecast lead time 01 to 09 for forecasts
#		00 valid for analyisis
# Fields:	Met office stash codes for required field(s)
#============================================#

#====== BADC server/login credentials =======#
our $username = "";
our $password = "";
#============================================#


###################################################
# DO NOT EDIT BELOW HERE UNLESS YOU KNOW WHAT YOU #
# ARE DOING!!                                     #
###################################################

#============================== idiot checks =========================#
die "Error: incompatible convert and delete_grib options!" if (!$convert && $delete_pp);
mkdir $output_dir, 0777 unless -d $output_dir or die $!;
#=====================================================================#

my @years = ($year_start .. $year_end);
my @months;
my @days;

my $month_from;
my $month_to;
my $day_from;
my $day_to;

our $ftp;
makeFTPConnection();

foreach my $year (@years) {
  if($year==$year_start) {
    $month_from = $month_start;
  } else {
    $month_from = 1;
  }
  if($year==$year_end) {
    $month_to = $month_end;
  } else {
    $month_to = 12;
  }
  @months = ($month_from .. $month_to);
  
  
  foreach my $month (@months) {
    if($year==$year_start && $month==$month_start) {
      $day_from = $day_start;
    } else {
      $day_from = 1;
    }
    if($year==$year_end && $month==$month_end) {
      $day_to = $day_end;
    } else {
      switch($month) {
	case [2] {
	  if($year%4==0) {
	    $day_to = 29;
	  } else {
	    $day_to = 28;
	  }
	}
	case [4,6,9,11] {$day_to = 30;}
	else {$day_to = 31;}
	}
    }
    @days = ($day_from .. $day_to);
    
    $month = "0$month" if(length($month)<2);
    
    foreach my $day (@days) {
      
      $day = "0$day" if(length($day)<2);
      
      foreach my $hour (@hours) {
	foreach my $field (@fields) {
	  foreach my $step (@steps) {
	    getGribFile($year,$month,$day,$hour,$stream,$field,$step);
	  }
	}
      }
      
    }
    
  }
    
}

$ftp->quit;

my $working_dir = getcwd;
my @convert_to_netCDF = ("conv2nc.tcl -i $output_dir*.pp -o $output_dir$nc_filename");

if($convert) {
  if(system(@convert_to_netCDF)==0) {
    my @lof = ($output_dir."*.pp");
    if($delete_pp) {
      unlink(<@lof>) or warn "$!";
    }
  } elsif($delete_pp) {
    print "NetCDF conversion may have failed, not removing pp files.";
  }
}



#############################################################################################




sub getGribFile {
  my ($year,$month,$day,$hour,$stream,$field,$step) = @_;  
    
  my $dataurl = "/badc/ukmo-um/data/nae/na/$stream/$year/$month/$day/na$stream$year$month$day$hour"."_$field"."_$step.pp";
  my $outname = "na$stream$year$month$day$hour"."_$field"."_$step.pp";

  print "Getting $dataurl\n";
  $ftp->get($dataurl,$output_dir.$outname)
    or do {print " --> Warning: ", $ftp->message."\n"; Time::HiRes::usleep(100000);}
    
}

sub makeFTPConnection {
  my $server = "ftp.badc.rl.ac.uk";
  
  $ftp = Net::FTP->new($server, Debug => 0)
    or die "Error: could not make ftp connection to $server.\nMessage: $@";
    
  $ftp->login($username,$password)
    or die "Error: could not login to $server with $username/$password: ", $ftp->message;
    
  $ftp->binary;
}

	

